"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectOrList = exports.ChoiceNode = void 0;
const Node_1 = require("./Node");
const ListNode_1 = require("./ListNode");
/**
 * Node that allows multiple types
 */
exports.ChoiceNode = (choices) => {
    const isValid = (choice, value) => {
        switch (choice) {
            case 'list': return value instanceof Array;
            case 'object': return typeof value === 'object' && !(value instanceof Array);
            default: return typeof value === choice;
        }
    };
    const activeChoice = (value) => {
        const index = choices.map(choice => isValid(choice[0], value)).indexOf(true);
        if (index === -1)
            return undefined;
        return choices[index];
    };
    return {
        ...Node_1.Base,
        default: () => choices[0][1].default(),
        render(path, value, view, options) {
            var _a, _b;
            const choice = (_a = activeChoice(value)) !== null && _a !== void 0 ? _a : choices[0];
            let inject = choices.map(c => {
                if (c[0] === choice[0]) {
                    return `<button class="selected" disabled>${path.push(c[0]).locale()}</button>`;
                }
                const buttonId = view.registerClick(el => {
                    view.model.set(path, c[2](value));
                });
                return `<button data-id="${buttonId}">${path.push(c[0]).locale()}</button>`;
            }).join('');
            return (_b = choice[1]) === null || _b === void 0 ? void 0 : _b.render(path, value, view, { ...options, hideLabel: false, inject: inject });
        },
        validate(path, value, errors) {
            const choice = activeChoice(value);
            if (choice === undefined) {
                return value;
            }
            return choice[1].validate(path, value, errors);
        }
    };
};
exports.ObjectOrList = (node) => {
    return exports.ChoiceNode([
        ['object', node, v => v[0]],
        ['list', ListNode_1.ListNode(node), v => [v]]
    ]);
};
