"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectNode = exports.Case = exports.Switch = void 0;
const Node_1 = require("./Node");
const Errors_1 = require("../model/Errors");
exports.Switch = Symbol('switch');
exports.Case = Symbol('case');
exports.ObjectNode = (fields, config) => {
    const { [exports.Switch]: filter, [exports.Case]: cases, ...defaultFields } = fields;
    const getActiveFields = (path, model) => {
        if (filter === undefined)
            return defaultFields;
        const switchValue = filter(path.withModel(model)).get();
        const activeCase = cases[switchValue];
        return { ...defaultFields, ...activeCase };
    };
    const renderFields = (path, value, view) => {
        value = value !== null && value !== void 0 ? value : {};
        const activeFields = getActiveFields(path, view.model);
        return Object.keys(activeFields).map(f => {
            if (!activeFields[f].enabled(path, view.model))
                return '';
            return activeFields[f].render(path.push(f), value[f], view);
        }).join('');
    };
    return ({
        ...Node_1.Base,
        default: () => ({}),
        transform(path, value, view) {
            if (value === undefined || value === null || typeof value !== 'object') {
                return value;
            }
            let res = {};
            const activeFields = getActiveFields(path, view.model);
            Object.keys(activeFields).forEach(f => {
                return res[f] = activeFields[f].transform(path.push(f), value[f], view);
            });
            return res;
        },
        render(path, value, view, options) {
            var _a;
            return `<div class="node object-node"${(config === null || config === void 0 ? void 0 : config.category) ? `data-category="${config === null || config === void 0 ? void 0 : config.category}"` : ''}>
        ${(options === null || options === void 0 ? void 0 : options.hideLabel) ? `` : `<div class="node-header" ${path.error()}>
          ${(options === null || options === void 0 ? void 0 : options.removeId) ? `
            <button class="remove" data-id="${options === null || options === void 0 ? void 0 : options.removeId}">
              ${(options === null || options === void 0 ? void 0 : options.removeLabel) ? options === null || options === void 0 ? void 0 : options.removeLabel : ''}
            </button>
          ` : ``}
          ${(options === null || options === void 0 ? void 0 : options.removeLabel) ? `` : `
            ${(options === null || options === void 0 ? void 0 : options.collapse) || (config === null || config === void 0 ? void 0 : config.collapse) ? value === undefined ? `
              <label class="collapse closed" data-id="${view.registerClick(() => view.model.set(path, this.default()))}">
                ${path.locale()}
              </label>
            ` : `
              <label class="collapse open" data-id="${view.registerClick(() => view.model.set(path, undefined))}">
                ${path.locale()}
              </label>
            ` : `
              <label>${path.locale()}</label>
            `}
          `}
          ${(_a = options === null || options === void 0 ? void 0 : options.inject) !== null && _a !== void 0 ? _a : ''}
        </div>`}
        ${(typeof value !== 'object' && value !== undefined) || (((options === null || options === void 0 ? void 0 : options.collapse) || (config === null || config === void 0 ? void 0 : config.collapse)) && value === undefined) ? `` : `
          <div class="node-body">
            ${renderFields(path, value, view)}
          </div>
        `}
      </div>`;
        },
        validate(path, value, errors) {
            value = value !== null && value !== void 0 ? value : {};
            if (typeof value !== 'object') {
                errors.add(path, 'error.expected_object');
                return value;
            }
            let res = {};
            let activeFields = defaultFields;
            if (filter) {
                const filterPath = filter(path);
                let switchValue = filterPath.get();
                if (path.equals(filterPath)) {
                    const filterField = filterPath.last();
                    defaultFields[filterField].validate(path.push(filterField), value[filterField], new Errors_1.Errors());
                }
                activeFields = { ...activeFields, ...cases[switchValue] };
            }
            const activeKeys = Object.keys(activeFields);
            const forcedKeys = activeKeys.filter(k => activeFields[k].force());
            const keys = new Set([...forcedKeys, ...Object.keys(value)]);
            keys.forEach(k => {
                res[k] = activeKeys.includes(k) ? activeFields[k].validate(path.push(k), value[k], errors) : value[k];
            });
            return res;
        }
    });
};
